/*!  \file
 * \brief Declaration of the ApplicationWindow main window class for our application
 *
 * Programmierpraktikum Computergrafik (CGP): Aufgabe 1 - "Ui, OpenGL!"
 *
 * Created by Christian Vogelgsang <Vogelgsang@informatik.uni-erlangen.de>,
 * changed by Martin Aumueller <aumueller@uni-koeln.de>
 */


/*! \file ui_ApplicationWindow.h
 * \brief Ui::ApplicationWindow class generated by Qt Designer
 */

/*! \class Ui::ApplicationWindow
 * \brief UI class generated by Qt Designer
 */

/*! \class Ui_ApplicationWindow
 * \brief UI class generated by Qt Designer
 */

#ifndef APPWIN_H
#define APPWIN_H

// Qt include files
#include <QMainWindow>
#include "ui_ApplicationWindow.h"

class UserParameterDialog;
class GLFrame;

/*! Declaration of ApplicationWindow the main application window.
  This ApplicationWindow contains a GLFrame for rendering the 3D object
  and an UserParameterDialog for selecting various render options.
  You can interact with the object by moving the mouse over
  the rendering area. */

//! application main window
class ApplicationWindow : public QMainWindow
{
    // place this macro here to include the standard Qt object
    // defitions
    Q_OBJECT

    public:

        //! create an application window and pass the application object
        ApplicationWindow();

        //! destroy application window
        virtual ~ApplicationWindow();

    signals:
        //! reset camera position and orientation to default values
        void resetCam();
        //! reset light position to default values
        void resetLight();
        //! request to open another model
        void openFile(const QString &filename) const;

        //! a different shader was selected
        void shader(int shader) const;
        //! GL draw mode was changed
        void glMode(int mode) const;
        //! render mode was changed
        void renderMode(int mode) const;
        //! material was changed
        void material(int material) const;

    protected:
        //! initialize application state
        void initState() const;

        //! our main widget in the application is a OGLCanvas
        GLFrame *m_frame;

        //! a pointer to the options dialog
        UserParameterDialog *m_userParamDialog;

        //! timer for triggering continuous re-rendering
        QTimer *m_trigger;

        //! designer ui
        Ui::ApplicationWindow ui;

        private slots:

        // ----- Slot Definition for our Application -----

        //! slot for showing dialog for modifying user shader parameter
        void userParamDialog();

        //! slot for File/Quit menu
        void fileQuit();

        //! slot to update statusbar message
        void updateMessage(const QString& message);

        //! show a file browser and load model
        void loadFile();

        //! load model from file
        void loadFile(QString filename);

        //! update FPS display
        void updateFps();

        //! start and stop render timer
        void animate(bool on);

        //! deduce render mode from menu state
        int getRenderMode() const;

        //! deduce GL mode from menu state
        int getGlMode() const;

        //! deduce shader from menu state
        int getShader() const;

        //! deduce material from menu state
        int getMaterial() const;

        //! make sure that GL mode signal is triggerd
        void glModeChanged() const;
        //! make sure that render mode signal is triggerd
        void renderModeChanged() const;
        //! make sure that shader signal is triggerd
        void shaderChanged() const;
        //! make sure that material signal is triggerd
        void materialChanged() const;

private:
        //! store programmatically created actions corresponding to material definition
        std::vector<QAction *> m_materialActions;
        //! store status bar message
        QString m_message;
        //! revert to default message if time is up
        int m_secondsToDisplay;
};

#endif
